﻿using System;
using System.Threading;
using System.Windows.Data;
using System.Windows.Media;
using System.ComponentModel;
using System.Collections.ObjectModel;

namespace FasterUltraGrid
{
	[ValueConversion(typeof(object), typeof(string))]
	public class ColorBrushConverter : IValueConverter
	{
		public ColorBrushConverter()
		{

		}

		private string[] strList = { 
											 Colors.AliceBlue.ToString(),
											 Colors.AntiqueWhite.ToString(),
											 Colors.Aqua.ToString(), 
											 Colors.Aquamarine.ToString(),
											 Colors.Azure.ToString(), 
											 Colors.Beige.ToString(), 
											 Colors.Bisque.ToString(), 
											 Colors.BlanchedAlmond.ToString(), 
											 Colors.Blue.ToString(),
											 Colors.Brown.ToString(),
											 Colors.Brown.ToString(),
											 Colors.Brown.ToString()
								};
	
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			if (value is double)
			{
				int index = (int)(Math.Ceiling(((double)value)/10) % 10);
				return strList[index];
			}
			else
			{
				return Binding.DoNothing;
			}
		}

		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			return null;
		}
	}


	public class CarsBusinessLogic
	{
		private ObservableCollection<Car> Cars = null;
		public string[] Makes = { "Dodge", "Ford", "BMW", "Toyota", "Maruti", "Fiat" };
		public string[] Models = { "D1", "F1", "B1", "T1", "M1", "F2" };
		private Timer changeTimer;
		private Random random;
		private int index = 0;

		public ObservableCollection<Car> GetCars()
		{
			return Cars;
		}

		public CarsBusinessLogic()
		{
			random = new Random((int)DateTime.Today.Ticks);
			InitCars();
		}

		public void Start()
		{
			changeTimer = new Timer(new TimerCallback((o) =>
			{
				lock (this)
				{
					Cars[index].BasePrice = (Cars[index].BasePrice + 10) % 100;
					index = ++index % 100;
				}
			}),null,0,1);
		}

		private void InitCars()
		{
			Cars = new ObservableCollection<Car>();
			for (double i = 0; i < 100; i++)
			{
				Cars.Add(new Car(Makes[(int)i % 6], Models[(int)i % 6], i % 100, i));
			}
		}

		public void Stop()
		{
			changeTimer.Dispose();
			changeTimer = null;
		}
	}

	public class Car : INotifyPropertyChanged
	{
		string m_make;
		string m_model;
		double m_baseprice;
		double m_mileage;

		public Car()
		{
		}

		public Car(string make, string model, double baseprice, double mileage)
		{
			this.Make = make;
			this.Model = model;
			this.BasePrice = baseprice;
			this.Mileage = mileage;
		}

		public string Make
		{
			get { return m_make; }
			set
			{
				if (m_make != value)
				{
					m_make = value;
					OnPropertyChanged("Make");
				}
			}
		}

		public string Model
		{
			get { return m_model; }
			set
			{
				if (m_model != value)
				{
					m_model = value;
					OnPropertyChanged("Model");
				}
			}
		}

		public double BasePrice
		{
			get { return m_baseprice; }
			set
			{
				if (m_baseprice != value)
				{
					m_baseprice = value;
					OnPropertyChanged("BasePrice");
				}
			}
		}

		public double Mileage
		{
			get { return m_mileage; }
			set
			{
				if (m_mileage != value)
				{
					m_mileage = value;
					OnPropertyChanged("Mileage");
				}
			}
		}

		#region INotifyPropertyChanged Members

		public event PropertyChangedEventHandler PropertyChanged;

		private void OnPropertyChanged(String info)
		{
			if (PropertyChanged != null)
			{
				PropertyChanged(this, new PropertyChangedEventArgs(info));
			}
		}
		#endregion
	}
}
